@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 575 $
- * $Date: 2013-03-04 23:01:14 +0100 (Mon, 04 Mar 2013) $
+ * $Revision: 633 $
+ * $Date: 2013-04-17 22:21:46 +0200 (Wed, 17 Apr 2013) $
  *
  */
 
@@ -72,12 +72,8 @@ protected:
 
 						Connection(SOCKET iSocket, bool bTLS);
 	void				ReportError(const char* szMsgPrefix, const char* szMsgArg, bool PrintErrCode, int herrno);
-	virtual bool 		DoConnect();
-	virtual bool		DoDisconnect();
-	int					DoBind();
-	int					DoWriteLine(const char* pBuffer);
-	char*				DoReadLine(char* pBuffer, int iSize, int* pBytesRead);
-	SOCKET				DoAccept();
+	bool				DoConnect();
+	bool				DoDisconnect();
 #ifndef HAVE_GETADDRINFO
 	unsigned int		ResolveHostAddr(const char* szHost);
 #endif
@@ -92,9 +88,9 @@ public:
 	virtual 			~Connection();
 	static void			Init();
 	static void			Final();
-	bool 				Connect();
-	bool				Disconnect();
-	int					Bind();
+	virtual bool 		Connect();
+	virtual bool		Disconnect();
+	bool				Bind();
 	bool				Send(const char* pBuffer, int iSize);
 	bool				Recv(char* pBuffer, int iSize);
 	int					TryRecv(char* pBuffer, int iSize);
