@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 555 $
- * $Date: 2013-02-06 23:04:50 +0100 (Wed, 06 Feb 2013) $
+ * $Revision: 658 $
+ * $Date: 2013-04-30 22:12:07 +0200 (Tue, 30 Apr 2013) $
  *                                                          
  */
 
@@ -56,7 +56,7 @@ public:
 	bool				LoadDownloadQueue(DownloadQueue* pDownloadQueue);
 	bool				SaveFile(FileInfo* pFileInfo);
 	bool				LoadArticles(FileInfo* pFileInfo);
-	bool				DiscardDownloadQueue();
+	void				DiscardDownloadQueue();
 	bool				DiscardFile(FileInfo* pFileInfo);
 	void				CleanupTempDir(DownloadQueue* pDownloadQueue);
 };
