@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 572 $
- * $Date: 2013-03-04 20:55:36 +0100 (Mon, 04 Mar 2013) $
+ * $Revision: 708 $
+ * $Date: 2013-06-13 22:51:38 +0200 (Thu, 13 Jun 2013) $
  *
  */
 
@@ -34,10 +34,10 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <cctype>
-#include <cstdio>
-#include <map>
+#include <stdio.h>
+#include <ctype.h>
 #include <sys/stat.h>
+#include <map>
 
 #include "nzbget.h"
 #include "DownloadInfo.h"
@@ -117,6 +117,59 @@ void NZBParameterList::SetParameter(const char* szName, const char* szValue)
 }
 
 
+ScriptStatus::ScriptStatus(const char* szName, EStatus eStatus)
+{
+	m_szName = strdup(szName);
+	m_eStatus = eStatus;
+}
+
+ScriptStatus::~ScriptStatus()
+{
+	if (m_szName)
+	{
+		free(m_szName);
+	}
+}
+
+
+ScriptStatusList::~ScriptStatusList()
+{
+	Clear();
+}
+
+void ScriptStatusList::Clear()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+	clear();
+}
+
+void ScriptStatusList::Add(const char* szScriptName, ScriptStatus::EStatus eStatus)
+{
+	push_back(new ScriptStatus(szScriptName, eStatus));
+}
+
+ScriptStatus::EStatus ScriptStatusList::CalcTotalStatus()
+{
+	ScriptStatus::EStatus eStatus = ScriptStatus::srNone;
+
+	for (iterator it = begin(); it != end(); it++)
+	{
+		ScriptStatus* pScriptStatus = *it;
+		// Failure-Status overrides Success-Status
+		if ((pScriptStatus->GetStatus() == ScriptStatus::srSuccess && eStatus == ScriptStatus::srNone) ||
+			(pScriptStatus->GetStatus() == ScriptStatus::srFailure))
+		{
+			eStatus = pScriptStatus->GetStatus();
+		}
+	}
+	
+	return eStatus;
+}
+
+
 NZBInfo::NZBInfo()
 {
 	debug("Creating NZBInfo");
@@ -133,8 +186,8 @@ NZBInfo::NZBInfo()
 	m_eRenameStatus = rsNone;
 	m_eParStatus = psNone;
 	m_eUnpackStatus = usNone;
+	m_eCleanupStatus = csNone;
 	m_eMoveStatus = msNone;
-	m_eScriptStatus = srNone;
 	m_bDeleted = false;
 	m_bParCleanup = false;
 	m_bCleanupDisk = false;
@@ -206,6 +259,15 @@ void NZBInfo::Release()
 	}
 }
 
+void NZBInfo::SetID(int iID)
+{
+	m_iID = iID;
+	if (m_iIDGen < m_iID)
+	{
+		m_iIDGen = m_iID;
+	}
+}
+
 void NZBInfo::ClearCompletedFiles()
 {
 	for (Files::iterator it = m_completedFiles.begin(); it != m_completedFiles.end(); it++)
@@ -337,12 +399,9 @@ void NZBInfo::BuildFinalDirName(char* szFinalDirBuf, int iBufSize)
 		strncpy(szFinalDirBuf, szBuffer, iBufSize);
 	}
 
-	if (g_pOptions->GetAppendNZBDir())
-	{
-		snprintf(szBuffer, 1024, "%s%s", szFinalDirBuf, GetName());
-		szBuffer[1024-1] = '\0';
-		strncpy(szFinalDirBuf, szBuffer, iBufSize);
-	}
+	snprintf(szBuffer, 1024, "%s%s", szFinalDirBuf, GetName());
+	szBuffer[1024-1] = '\0';
+	strncpy(szFinalDirBuf, szBuffer, iBufSize);
 }
 
 void NZBInfo::SetParameter(const char* szName, const char* szValue)
@@ -460,6 +519,7 @@ FileInfo::FileInfo()
 	m_szSubject = NULL;
 	m_szFilename = NULL;
 	m_szOutputFilename = NULL;
+	m_pMutexOutputFile = NULL;
 	m_bFilenameConfirmed = false;
 	m_lSize = 0;
 	m_lRemainingSize = 0;
@@ -472,6 +532,7 @@ FileInfo::FileInfo()
 	m_iPriority = 0;
 	m_bExtraPriority = false;
 	m_iActiveDownloads = 0;
+	m_bAutoDeleted = false;
 	m_iIDGen++;
 	m_iID = m_iIDGen;
 }
@@ -492,6 +553,10 @@ FileInfo::~ FileInfo()
 	{
 		free(m_szOutputFilename);
 	}
+	if (m_pMutexOutputFile)
+	{
+		delete m_pMutexOutputFile;
+	}
 
 	for (Groups::iterator it = m_Groups.begin(); it != m_Groups.end() ;it++)
 	{
@@ -516,9 +581,9 @@ void FileInfo::ClearArticles()
 	m_Articles.clear();
 }
 
-void FileInfo::SetID(int s)
+void FileInfo::SetID(int iID)
 {
-	m_iID = s;
+	m_iID = iID;
 	if (m_iIDGen < m_iID)
 	{
 		m_iIDGen = m_iID;
@@ -556,12 +621,12 @@ void FileInfo::MakeValidFilename()
 
 void FileInfo::LockOutputFile()
 {
-	m_mutexOutputFile.Lock();
+	m_pMutexOutputFile->Lock();
 }
 
 void FileInfo::UnlockOutputFile()
 {
-	m_mutexOutputFile.Unlock();
+	m_pMutexOutputFile->Unlock();
 }
 
 void FileInfo::SetOutputFilename(const char* szOutputFilename)
@@ -573,6 +638,21 @@ void FileInfo::SetOutputFilename(const char* szOutputFilename)
 	m_szOutputFilename = strdup(szOutputFilename);
 }
 
+void FileInfo::SetActiveDownloads(int iActiveDownloads)
+{
+	m_iActiveDownloads = iActiveDownloads;
+
+	if (m_iActiveDownloads > 0 && !m_pMutexOutputFile)
+	{
+		m_pMutexOutputFile = new Mutex();
+	}
+	else if (m_iActiveDownloads == 0 && m_pMutexOutputFile)
+	{
+		delete m_pMutexOutputFile;
+		m_pMutexOutputFile = NULL;
+	}
+}
+
 bool FileInfo::IsDupe(const char* szFilename)
 {
 	char fileName[1024];
@@ -621,16 +701,11 @@ PostInfo::PostInfo()
 	debug("Creating PostInfo");
 
 	m_pNZBInfo = NULL;
-	m_szParFilename = NULL;
 	m_szInfoName = NULL;
 	m_bWorking = false;
 	m_bDeleted = false;
-	m_eRenameStatus = rsNone;
-	m_eParStatus = psNone;
-	m_eUnpackStatus = usNone;
-	m_eRequestParCheck = rpNone;
+	m_bRequestParCheck = false;
 	m_bRequestParRename = false;
-	m_eScriptStatus = srNone;
 	m_szProgressLabel = strdup("");
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
@@ -648,10 +723,6 @@ PostInfo::~ PostInfo()
 {
 	debug("Destroying PostInfo");
 
-	if (m_szParFilename)
-	{
-		free(m_szParFilename);
-	}
 	if (m_szInfoName)
 	{
 		free(m_szInfoName);
@@ -683,11 +754,6 @@ void PostInfo::SetNZBInfo(NZBInfo* pNZBInfo)
 	m_pNZBInfo->AddReference();
 }
 
-void PostInfo::SetParFilename(const char* szParFilename)
-{
-	m_szParFilename = strdup(szParFilename);
-}
-
 void PostInfo::SetInfoName(const char* szInfoName)
 {
 	m_szInfoName = strdup(szInfoName);
@@ -837,9 +903,9 @@ void UrlInfo::SetURL(const char* szURL)
 	m_szURL = strdup(szURL);
 }
 
-void UrlInfo::SetID(int s)
+void UrlInfo::SetID(int iID)
 {
-	m_iID = s;
+	m_iID = iID;
 	if (m_iIDGen < m_iID)
 	{
 		m_iIDGen = m_iID;
@@ -919,9 +985,9 @@ HistoryInfo::~HistoryInfo()
 	}
 }
 
-void HistoryInfo::SetID(int s)
+void HistoryInfo::SetID(int iID)
 {
-	m_iID = s;
+	m_iID = iID;
 	if (m_iIDGen < m_iID)
 	{
 		m_iIDGen = m_iID;
