@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2009 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 512 $
- * $Date: 2012-11-20 21:42:00 +0100 (Tue, 20 Nov 2012) $
+ * $Revision: 677 $
+ * $Date: 2013-05-14 22:20:52 +0200 (Tue, 14 May 2013) $
  *
  */
 
@@ -38,7 +38,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <stdarg.h>
-#include <cstdio>
+#include <stdio.h>
 
 #include "nzbget.h"
 #include "Options.h"
