@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 424 $
- * $Date: 2012-06-23 20:58:56 +0200 (Sat, 23 Jun 2012) $
+ * $Revision: 711 $
+ * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
  *
  */
 
@@ -121,16 +121,6 @@ void NZBFile::DetachFileInfos()
     m_FileInfos.clear();
 }
 
-NZBFile* NZBFile::CreateFromBuffer(const char* szFileName, const char* szCategory, const char* szBuffer, int iSize)
-{
-	return Create(szFileName, szCategory, szBuffer, iSize, true);
-}
-
-NZBFile* NZBFile::CreateFromFile(const char* szFileName, const char* szCategory)
-{
-	return Create(szFileName, szCategory, NULL, 0, false);
-}
-
 void NZBFile::AddArticle(FileInfo* pFileInfo, ArticleInfo* pArticleInfo)
 {
 	// make Article-List big enough
@@ -357,7 +347,7 @@ void NZBFile::ProcessFilenames()
 }
 
 #ifdef WIN32
-NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory, const char* szBuffer, int iSize, bool bFromBuffer)
+NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory)
 {
     CoInitialize(NULL);
 
@@ -374,21 +364,15 @@ NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory, const c
 	doc->put_resolveExternals(VARIANT_FALSE);
 	doc->put_validateOnParse(VARIANT_FALSE);
 	doc->put_async(VARIANT_FALSE);
-	VARIANT_BOOL success;
-	if (bFromBuffer)
-	{
-		success = doc->loadXML(szBuffer);
-	}
-	else
-	{
-		// filename needs to be properly encoded
-		char* szURL = (char*)malloc(strlen(szFileName)*3 + 1);
-		EncodeURL(szFileName, szURL);
-		debug("url=\"%s\"", szURL);
-		_variant_t v(szURL);
-		free(szURL);
-		success = doc->load(v);
-	} 
+
+	// filename needs to be properly encoded
+	char* szURL = (char*)malloc(strlen(szFileName)*3 + 1);
+	EncodeURL(szFileName, szURL);
+	debug("url=\"%s\"", szURL);
+	_variant_t v(szURL);
+	free(szURL);
+
+	VARIANT_BOOL success = doc->load(v);
 	if (success == VARIANT_FALSE)
 	{
 		_bstr_t r(doc->GetparseError()->reason);
@@ -482,11 +466,14 @@ bool NZBFile::ParseNZB(IUnknown* nzb)
 			int partNumber = atoi(number);
 			int lsize = atoi(bytes);
 
-			ArticleInfo* pArticle = new ArticleInfo();
-			pArticle->SetPartNumber(partNumber);
-			pArticle->SetMessageID(szId);
-			pArticle->SetSize(lsize);
-			AddArticle(pFileInfo, pArticle);
+			if (partNumber > 0)
+			{
+				ArticleInfo* pArticle = new ArticleInfo();
+				pArticle->SetPartNumber(partNumber);
+				pArticle->SetMessageID(szId);
+				pArticle->SetSize(lsize);
+				AddArticle(pFileInfo, pArticle);
+			}
 
             if (lsize > 0)
             {
@@ -501,7 +488,7 @@ bool NZBFile::ParseNZB(IUnknown* nzb)
 
 #else
 
-NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory, const char* szBuffer, int iSize, bool bFromBuffer)
+NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory)
 {
     NZBFile* pFile = new NZBFile(szFileName, szCategory);
 
@@ -512,18 +499,10 @@ NZBFile* NZBFile::Create(const char* szFileName, const char* szCategory, const c
 	SAX_handler.error = reinterpret_cast<errorSAXFunc>(SAX_error);
 	SAX_handler.getEntity = reinterpret_cast<getEntitySAXFunc>(SAX_getEntity);
 
-	int ret = 0;
 	pFile->m_bIgnoreNextError = false;
 
-	if (bFromBuffer)
-	{
-		ret = xmlSAXUserParseMemory(&SAX_handler, pFile, szBuffer, iSize);
-	}
-	else
-	{
-		ret = xmlSAXUserParseFile(&SAX_handler, pFile, szFileName);
-	}
-        
+	int ret = xmlSAXUserParseFile(&SAX_handler, pFile, szFileName);
+    
     if (ret == 0)
 	{
 		pFile->ProcessFilenames();
