@@ -2,7 +2,7 @@
  *  This file is part of nzbget
  *
  *  Copyright (C) 2004 Sven Henkel <sidddy@users.sourceforge.net>
- *  Copyright (C) 2007-2010 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 404 $
- * $Date: 2012-01-01 21:51:23 +0100 (Sun, 01 Jan 2012) $
+ * $Revision: 711 $
+ * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
  *
  */
 
@@ -66,12 +66,10 @@ private:
 	void				Parse_EndElement(const char *name);
 	void				Parse_Content(const char *buf, int len);
 #endif
-	static NZBFile*		Create(const char* szFileName, const char* szCategory, const char* szBuffer, int iSize, bool bFromBuffer);
 
 public:
 	virtual 			~NZBFile();
-	static NZBFile*		CreateFromBuffer(const char* szFileName, const char* szCategory, const char* szBuffer, int iSize);
-	static NZBFile*		CreateFromFile(const char* szFileName, const char* szCategory);
+	static NZBFile*		Create(const char* szFileName, const char* szCategory);
 	const char* 		GetFileName() const { return m_szFileName; }
 	FileInfos*			GetFileInfos() { return &m_FileInfos; }
 	NZBInfo*			GetNZBInfo() { return m_pNZBInfo; }
