@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2009 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 543 $
- * $Date: 2013-01-17 23:08:01 +0100 (Thu, 17 Jan 2013) $
+ * $Revision: 677 $
+ * $Date: 2013-05-14 22:20:52 +0200 (Tue, 14 May 2013) $
  *
  */
 
@@ -29,19 +29,20 @@
 #ifndef DISABLE_PARCHECK
 
 #include <deque>
+#include <string>
 
 #include "Thread.h"
-#include "Observer.h"
+#include "Log.h"
 
-class ParChecker : public Thread, public Subject
+class ParChecker : public Thread
 {
 public:
 	enum EStatus
 	{
-        psUndefined,
-		psWorking,
 		psFailed,
-		psFinished
+		psRepairPossible,
+		psRepaired,
+		psRepairNotNeeded
 	};
 
 	enum EStage
@@ -53,15 +54,17 @@ public:
 	};
 
 	typedef std::deque<char*>		FileList;
+	typedef std::deque<void*>		SourceList;
 	
 private:
 	char*				m_szInfoName;
-	char*				m_szParFilename;
+	char*				m_szDestDir;
+	char*				m_szNZBName;
+	const char*			m_szParFilename;
 	EStatus				m_eStatus;
 	EStage				m_eStage;
 	void*				m_pRepairer;	// declared as void* to prevent the including of libpar2-headers into this header-file
 	char*				m_szErrMsg;
-	bool				m_bRepairNotNeeded;
 	FileList			m_QueuedParFiles;
 	Mutex			 	m_mutexQueuedParFiles;
 	bool				m_bQueuedParFilesChanged;
@@ -74,12 +77,17 @@ private:
 	int					m_iFileProgress;
 	int					m_iStageProgress;
 	bool				m_bCancelled;
+	SourceList			m_sourceFiles;
 
+	EStatus				RunParCheck(const char* szParFilename);
+	void				WriteBrokenLog(EStatus eStatus);
 	void				Cleanup();
 	bool				LoadMorePars();
 	bool				CheckSplittedFragments();
 	bool				AddSplittedFragments(const char* szFilename);
 	bool				AddMissingFiles();
+	void				SaveSourceList();
+	void				DeleteLeftovers();
 	void				signal_filename(std::string str);
 	void				signal_progress(double progress);
 	void				signal_done(std::string str, int available, int total);
@@ -92,6 +100,8 @@ protected:
 	*/
 	virtual bool		RequestMorePars(int iBlockNeeded, int* pBlockFound) = 0;
 	virtual void		UpdateProgress() {}
+	virtual void		Completed() {}
+	virtual void		PrintMessage(Message::EKind eKind, const char* szFormat, ...) {}
 	EStage				GetStage() { return m_eStage; }
 	const char*			GetProgressLabel() { return m_szProgressLabel; }
 	int					GetFileProgress() { return m_iFileProgress; }
@@ -101,14 +111,12 @@ public:
 						ParChecker();
 	virtual				~ParChecker();
 	virtual void		Run();
+	void				SetDestDir(const char* szDestDir);
 	const char*			GetParFilename() { return m_szParFilename; }
-	void				SetParFilename(const char* szParFilename);
 	const char*			GetInfoName() { return m_szInfoName; }
 	void				SetInfoName(const char* szInfoName);
-	void				SetStatus(EStatus eStatus);
+	void				SetNZBName(const char* szNZBName);
 	EStatus				GetStatus() { return m_eStatus; }
-	const char*			GetErrMsg() { return m_szErrMsg; }
-	bool				GetRepairNotNeeded() { return m_bRepairNotNeeded; }
 	void				AddParFile(const char* szParFilename);
 	void				QueueChanged();
 	void				Cancel();
