@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 572 $
- * $Date: 2013-03-04 20:55:36 +0100 (Mon, 04 Mar 2013) $
+ * $Revision: 680 $
+ * $Date: 2013-05-15 18:45:36 +0200 (Wed, 15 May 2013) $
  *
  */
 
@@ -33,7 +33,8 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <fstream>
+#include <stdio.h>
+#include <stdarg.h>
 #ifdef WIN32
 #include <direct.h>
 #else
@@ -63,10 +64,34 @@ void ParCoordinator::PostParChecker::UpdateProgress()
 	m_pOwner->UpdateParCheckProgress();
 }
 
+void ParCoordinator::PostParChecker::PrintMessage(Message::EKind eKind, const char* szFormat, ...)
+{
+	char szText[1024];
+	va_list args;
+	va_start(args, szFormat);
+	vsnprintf(szText, 1024, szFormat, args);
+	va_end(args);
+	szText[1024-1] = '\0';
+
+	m_pOwner->PrintMessage(m_pPostInfo, eKind, "%s", szText);
+}
+
 void ParCoordinator::PostParRenamer::UpdateProgress()
 {
 	m_pOwner->UpdateParRenameProgress();
 }
+
+void ParCoordinator::PostParRenamer::PrintMessage(Message::EKind eKind, const char* szFormat, ...)
+{
+	char szText[1024];
+	va_list args;
+	va_start(args, szFormat);
+	vsnprintf(szText, 1024, szFormat, args);
+	va_end(args);
+	szText[1024-1] = '\0';
+	
+	m_pOwner->PrintMessage(m_pPostInfo, eKind, "%s", szText);
+}
 #endif
 
 ParCoordinator::ParCoordinator()
@@ -74,18 +99,9 @@ ParCoordinator::ParCoordinator()
 	debug("Creating ParCoordinator");
 
 #ifndef DISABLE_PARCHECK
-	m_ParCheckerObserver.m_pOwner = this;
-	m_ParChecker.Attach(&m_ParCheckerObserver);
-	m_ParChecker.m_pOwner = this;
-
-	m_ParRenamerObserver.m_pOwner = this;
-	m_ParRenamer.Attach(&m_ParRenamerObserver);
-	m_ParRenamer.m_pOwner = this;
-	
 	m_bStopped = false;
-
-	const char* szPostScript = g_pOptions->GetPostProcess();
-	m_bPostScript = szPostScript && strlen(szPostScript) > 0;
+	m_ParChecker.m_pOwner = this;
+	m_ParRenamer.m_pOwner = this;
 #endif
 }
 
@@ -129,10 +145,7 @@ void ParCoordinator::PausePars(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 		if (pFileInfo->GetNZBInfo() == pNZBInfo)
 		{
 			g_pQueueCoordinator->GetQueueEditor()->LockedEditEntry(pDownloadQueue, pFileInfo->GetID(), false, 
-				(g_pOptions->GetLoadPars() == Options::lpOne ||
-					(g_pOptions->GetLoadPars() == Options::lpNone && g_pOptions->GetParCheck()))
-				? QueueEditor::eaGroupPauseExtraPars : QueueEditor::eaGroupPauseAllPars,
-				0, NULL);
+				QueueEditor::eaGroupPauseExtraPars, 0, NULL);
 			break;
 		}
 	}
@@ -245,11 +258,11 @@ bool ParCoordinator::ParseParFilename(const char* szParFilename, int* iBaseNameL
  */
 void ParCoordinator::StartParCheckJob(PostInfo* pPostInfo)
 {
-	info("Checking pars for %s", pPostInfo->GetInfoName());
 	m_eCurrentJob = jkParCheck;
 	m_ParChecker.SetPostInfo(pPostInfo);
-	m_ParChecker.SetParFilename(pPostInfo->GetParFilename());
-	m_ParChecker.SetInfoName(pPostInfo->GetInfoName());
+	m_ParChecker.SetDestDir(pPostInfo->GetNZBInfo()->GetDestDir());
+	m_ParChecker.SetNZBName(pPostInfo->GetNZBInfo()->GetName());
+	m_ParChecker.PrintMessage(Message::mkInfo, "Checking pars for %s", pPostInfo->GetInfoName());
 	pPostInfo->SetWorking(true);
 	m_ParChecker.Start();
 }
@@ -259,11 +272,11 @@ void ParCoordinator::StartParCheckJob(PostInfo* pPostInfo)
  */
 void ParCoordinator::StartParRenameJob(PostInfo* pPostInfo)
 {
-	info("Checking renamed files for %s", pPostInfo->GetNZBInfo()->GetName());
 	m_eCurrentJob = jkParRename;
 	m_ParRenamer.SetPostInfo(pPostInfo);
 	m_ParRenamer.SetDestDir(pPostInfo->GetNZBInfo()->GetDestDir());
 	m_ParRenamer.SetInfoName(pPostInfo->GetNZBInfo()->GetName());
+	m_ParRenamer.PrintMessage(Message::mkInfo, "Checking renamed files for %s", pPostInfo->GetNZBInfo()->GetName());
 	pPostInfo->SetWorking(true);
 	m_ParRenamer.Start();
 }
@@ -322,106 +335,38 @@ bool ParCoordinator::AddPar(FileInfo* pFileInfo, bool bDeleted)
 	return bSameCollection;
 }
 
-void ParCoordinator::ParCheckerUpdate(Subject* Caller, void* Aspect)
+void ParCoordinator::ParCheckCompleted()
 {
-	if (m_ParChecker.GetStatus() == ParChecker::psFinished ||
-		m_ParChecker.GetStatus() == ParChecker::psFailed)
+	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
+
+	PostInfo* pPostInfo = m_ParChecker.GetPostInfo();
+
+	// Update ParStatus (accumulate result)
+	if ((m_ParChecker.GetStatus() == ParChecker::psRepaired ||
+		m_ParChecker.GetStatus() == ParChecker::psRepairNotNeeded) &&
+		pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped)
 	{
-		char szPath[1024];
-		strncpy(szPath, m_ParChecker.GetParFilename(), 1024);
-		szPath[1024-1] = '\0';
-		if (char* p = strrchr(szPath, PATH_SEPARATOR)) *p = '\0';
-
-		if (g_pOptions->GetCreateBrokenLog())
-		{
-			char szBrokenLogName[1024];
-			snprintf(szBrokenLogName, 1024, "%s%c_brokenlog.txt", szPath, (int)PATH_SEPARATOR);
-			szBrokenLogName[1024-1] = '\0';
-			
-			if (!m_ParChecker.GetRepairNotNeeded() || Util::FileExists(szBrokenLogName))
-			{
-				FILE* file = fopen(szBrokenLogName, "ab");
-				if (file)
-				{
-					if (m_ParChecker.GetStatus() == ParChecker::psFailed)
-					{
-						if (m_ParChecker.GetCancelled())
-						{
-							fprintf(file, "Repair cancelled for %s\n", m_ParChecker.GetInfoName());
-						}
-						else
-						{
-							fprintf(file, "Repair failed for %s: %s\n", m_ParChecker.GetInfoName(), m_ParChecker.GetErrMsg() ? m_ParChecker.GetErrMsg() : "");
-						}
-					}
-					else if (m_ParChecker.GetRepairNotNeeded())
-					{
-						fprintf(file, "Repair not needed for %s\n", m_ParChecker.GetInfoName());
-					}
-					else
-					{
-						if (g_pOptions->GetParRepair())
-						{
-							fprintf(file, "Successfully repaired %s\n", m_ParChecker.GetInfoName());
-						}
-						else
-						{
-							fprintf(file, "Repair possible for %s\n", m_ParChecker.GetInfoName());
-						}
-					}
-					fclose(file);
-				}
-				else
-				{
-					error("Could not open file %s", szBrokenLogName);
-				}
-			}
-		}
-
-		DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
-
-		PostInfo* pPostInfo = m_ParChecker.GetPostInfo();
-		pPostInfo->SetWorking(false);
-		if (pPostInfo->GetDeleted())
-		{
-			pPostInfo->SetStage(PostInfo::ptFinished);
-		}
-		else
-		{
-			pPostInfo->SetStage(PostInfo::ptQueued);
-		}
-
-		// Update ParStatus by NZBInfo (accumulate result)
-		if (m_ParChecker.GetStatus() == ParChecker::psFailed && !m_ParChecker.GetCancelled())
-		{
-			pPostInfo->SetParStatus(PostInfo::psFailure);
-			pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psFailure);
-		}
-		else if (m_ParChecker.GetStatus() == ParChecker::psFinished &&
-			(g_pOptions->GetParRepair() || m_ParChecker.GetRepairNotNeeded()))
-		{
-			pPostInfo->SetParStatus(PostInfo::psSuccess);
-			if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psNone)
-			{
-				pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psSuccess);
-			}
-		}
-		else
-		{
-			pPostInfo->SetParStatus(PostInfo::psRepairPossible);
-			if (pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure)
-			{
-				pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psRepairPossible);
-			}
-		}
-
-		if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
-		{
-			g_pDiskState->SaveDownloadQueue(pDownloadQueue);
-		}
-
-		g_pQueueCoordinator->UnlockQueue();
+		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psSuccess);
 	}
+	else if (m_ParChecker.GetStatus() == ParChecker::psRepairPossible &&
+		pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure)
+	{
+		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psRepairPossible);
+	}
+	else
+	{
+		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psFailure);
+	}
+
+	pPostInfo->SetWorking(false);
+	pPostInfo->SetStage(PostInfo::ptQueued);
+
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	{
+		g_pDiskState->SaveDownloadQueue(pDownloadQueue);
+	}
+
+	g_pQueueCoordinator->UnlockQueue();
 }
 
 /**
@@ -473,7 +418,7 @@ bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilenam
 			{
 				if (pBestBlockInfo->m_pFileInfo->GetPaused())
 				{
-					info("Unpausing %s%c%s for par-recovery", pNZBInfo->GetName(), (int)PATH_SEPARATOR, pBestBlockInfo->m_pFileInfo->GetFilename());
+					m_ParChecker.PrintMessage(Message::mkInfo, "Unpausing %s%c%s for par-recovery", pNZBInfo->GetName(), (int)PATH_SEPARATOR, pBestBlockInfo->m_pFileInfo->GetFilename());
 					pBestBlockInfo->m_pFileInfo->SetPaused(false);
 					pBestBlockInfo->m_pFileInfo->SetExtraPriority(true);
 				}
@@ -497,7 +442,7 @@ bool ParCoordinator::RequestMorePars(NZBInfo* pNZBInfo, const char* szParFilenam
 			BlockInfo* pBlockInfo = blocks.front();
 			if (pBlockInfo->m_pFileInfo->GetPaused())
 			{
-				info("Unpausing %s%c%s for par-recovery", pNZBInfo->GetName(), (int)PATH_SEPARATOR, pBlockInfo->m_pFileInfo->GetFilename());
+				m_ParChecker.PrintMessage(Message::mkInfo, "Unpausing %s%c%s for par-recovery", pNZBInfo->GetName(), (int)PATH_SEPARATOR, pBlockInfo->m_pFileInfo->GetFilename());
 				pBlockInfo->m_pFileInfo->SetPaused(false);
 				pBlockInfo->m_pFileInfo->SetExtraPriority(true);
 			}
@@ -615,9 +560,9 @@ void ParCoordinator::FindPars(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo,
 
 void ParCoordinator::UpdateParCheckProgress()
 {
-	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
+	g_pQueueCoordinator->LockQueue();
 
-	PostInfo* pPostInfo = pDownloadQueue->GetPostQueue()->front();
+	PostInfo* pPostInfo = m_ParChecker.GetPostInfo();
 	if (m_ParChecker.GetFileProgress() == 0)
 	{
 		pPostInfo->SetProgressLabel(m_ParChecker.GetProgressLabel());
@@ -654,7 +599,7 @@ void ParCoordinator::UpdateParCheckProgress()
 			if (iEstimatedRepairTime > g_pOptions->GetParTimeLimit() * 60)
 			{
 				debug("Estimated repair time %i seconds", iEstimatedRepairTime);
-				warn("Cancelling par-repair for %s, estimated repair time (%i minutes) exceeds allowed repair time", m_ParChecker.GetInfoName(), iEstimatedRepairTime / 60);
+				m_ParChecker.PrintMessage(Message::mkWarning, "Cancelling par-repair for %s, estimated repair time (%i minutes) exceeds allowed repair time", m_ParChecker.GetInfoName(), iEstimatedRepairTime / 60);
 				bParCancel = true;
 			}
 		}
@@ -701,50 +646,28 @@ void ParCoordinator::CheckPauseState(PostInfo* pPostInfo)
 	}
 }
 
-void ParCoordinator::ParRenamerUpdate(Subject* Caller, void* Aspect)
+void ParCoordinator::ParRenameCompleted()
 {
-	if (m_ParRenamer.GetStatus() == ParRenamer::psFinished ||
-		m_ParRenamer.GetStatus() == ParRenamer::psFailed)
+	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
+	
+	PostInfo* pPostInfo = m_ParRenamer.GetPostInfo();
+	pPostInfo->GetNZBInfo()->SetRenameStatus(m_ParRenamer.GetStatus() == ParRenamer::psSuccess ? NZBInfo::rsSuccess : NZBInfo::rsFailure);
+	pPostInfo->SetWorking(false);
+	pPostInfo->SetStage(PostInfo::ptQueued);
+	
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
 	{
-		DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
-		
-		PostInfo* pPostInfo = m_ParRenamer.GetPostInfo();
-		pPostInfo->SetWorking(false);
-		if (pPostInfo->GetDeleted())
-		{
-			pPostInfo->SetStage(PostInfo::ptFinished);
-		}
-		else
-		{
-			pPostInfo->SetStage(PostInfo::ptQueued);
-		}
-		
-		// Update ParStatus by NZBInfo
-		if (m_ParRenamer.GetStatus() == ParRenamer::psFailed && !m_ParRenamer.GetCancelled())
-		{
-			pPostInfo->SetRenameStatus(PostInfo::rsFailure);
-			pPostInfo->GetNZBInfo()->SetRenameStatus(NZBInfo::rsFailure);
-		}
-		else if (m_ParRenamer.GetStatus() == ParRenamer::psFinished)
-		{
-			pPostInfo->SetRenameStatus(PostInfo::rsSuccess);
-			pPostInfo->GetNZBInfo()->SetRenameStatus(NZBInfo::rsSuccess);
-		}
-		
-		if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
-		{
-			g_pDiskState->SaveDownloadQueue(pDownloadQueue);
-		}
-		
-		g_pQueueCoordinator->UnlockQueue();
+		g_pDiskState->SaveDownloadQueue(pDownloadQueue);
 	}
+	
+	g_pQueueCoordinator->UnlockQueue();
 }
 
 void ParCoordinator::UpdateParRenameProgress()
 {
-	DownloadQueue* pDownloadQueue = g_pQueueCoordinator->LockQueue();
+	g_pQueueCoordinator->LockQueue();
 	
-	PostInfo* pPostInfo = pDownloadQueue->GetPostQueue()->front();
+	PostInfo* pPostInfo = m_ParRenamer.GetPostInfo();
 	pPostInfo->SetProgressLabel(m_ParRenamer.GetProgressLabel());
 	pPostInfo->SetStageProgress(m_ParRenamer.GetStageProgress());
 	time_t tCurrent = time(NULL);
@@ -765,4 +688,39 @@ void ParCoordinator::UpdateParRenameProgress()
 	CheckPauseState(pPostInfo);
 }
 
+void ParCoordinator::PrintMessage(PostInfo* pPostInfo, Message::EKind eKind, const char* szFormat, ...)
+{
+	char szText[1024];
+	va_list args;
+	va_start(args, szFormat);
+	vsnprintf(szText, 1024, szFormat, args);
+	va_end(args);
+	szText[1024-1] = '\0';
+
+	pPostInfo->AppendMessage(eKind, szText);
+
+	switch (eKind)
+	{
+		case Message::mkDetail:
+			detail("%s", szText);
+			break;
+
+		case Message::mkInfo:
+			info("%s", szText);
+			break;
+
+		case Message::mkWarning:
+			warn("%s", szText);
+			break;
+
+		case Message::mkError:
+			error("%s", szText);
+			break;
+
+		case Message::mkDebug:
+			debug("%s", szText);
+			break;
+	}
+}
+
 #endif
