@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 570 $
- * $Date: 2013-03-01 21:32:17 +0100 (Fri, 01 Mar 2013) $
+ * $Revision: 711 $
+ * $Date: 2013-06-17 22:39:46 +0200 (Mon, 17 Jun 2013) $
  *
  */
 
@@ -31,7 +31,6 @@
 #include "Thread.h"
 #include "Observer.h"
 #include "DownloadInfo.h"
-#include "Scanner.h"
 #include "ParCoordinator.h"
 
 class PrePostProcessor : public Thread
@@ -45,7 +44,8 @@ public:
 		eaPostDelete,
 		eaHistoryDelete,
 		eaHistoryReturn,
-		eaHistoryProcess
+		eaHistoryProcess,
+		eaHistorySetParameter
 	};
 
 private:
@@ -71,18 +71,16 @@ private:
 	PostParCoordinator	m_ParCoordinator;
 	QueueCoordinatorObserver	m_QueueCoordinatorObserver;
 	bool				m_bHasMoreJobs;
-	bool				m_bPostScript;
 	bool				m_bSchedulerPauseChanged;
 	bool				m_bSchedulerPause;
 	bool				m_bPostPause;
-	Scanner				m_Scanner;
 	const char*			m_szPauseReason;
 
 	bool				IsNZBFileCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, 
-							bool bIgnorePausedPars, bool bCheckPostQueue, bool bAllowOnlyOneDeleted);
+							bool bIgnorePausedPars, bool bAllowOnlyOneDeleted);
 	void				CheckPostQueue();
 	void				JobCompleted(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo);
-	void				StartProcessJob(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo);
+	void				StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo);
 	void				SaveQueue(DownloadQueue* pDownloadQueue);
 	void				SanitisePostQueue(PostQueue* pPostQueue);
 	void				CheckDiskSpace();
@@ -95,13 +93,14 @@ private:
 	void				NZBDownloaded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo);
 	void				NZBDeleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo);
 	void				NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, bool bSaveQueue);
-	bool				CreatePostJobs(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, bool bParCheck, bool bUnpackOrScript, bool bAddTop);
 	void				DeleteQueuedFile(const char* szQueuedFile);
 	NZBInfo*			MergeGroups(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo);
 	bool				PostQueueMove(IDList* pIDList, EEditAction eAction, int iOffset);
 	bool				PostQueueDelete(IDList* pIDList);
-	bool				HistoryDelete(IDList* pIDList);
-	bool				HistoryReturn(IDList* pIDList, bool bReprocess);
+	bool				HistoryEdit(IDList* pIDList, EEditAction eAction, int iOffset, const char* szText);
+	void				HistoryDelete(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory, HistoryInfo* pHistoryInfo);
+	void				HistoryReturn(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory, HistoryInfo* pHistoryInfo, bool bReprocess);
+	void				HistorySetParameter(HistoryInfo* pHistoryInfo, const char* szText);
 	void				Cleanup();
 	FileInfo*			GetQueueGroup(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo);
 	void				CheckHistory();
@@ -114,8 +113,7 @@ public:
 	virtual void		Stop();
 	void				QueueCoordinatorUpdate(Subject* Caller, void* Aspect);
 	bool				HasMoreJobs() { return m_bHasMoreJobs; }
-	void				ScanNZBDir(bool bSyncMode);
-	bool				QueueEditList(IDList* pIDList, EEditAction eAction, int iOffset);
+	bool				QueueEditList(IDList* pIDList, EEditAction eAction, int iOffset, const char* szText);
 };
 
 #endif
