@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 592 $
- * $Date: 2013-03-11 20:54:14 +0100 (Mon, 11 Mar 2013) $
+ * $Revision: 695 $
+ * $Date: 2013-05-26 22:42:15 +0200 (Sun, 26 May 2013) $
  *
  */
 
@@ -63,6 +63,7 @@ private:
 	QueueEditor				m_QueueEditor;
 	Mutex			 		m_mutexDownloadQueue;
 	bool					m_bHasMoreJobs;
+	int						m_iDownloadsLimit;
 
 	// statistics
 	static const int		SPEEDMETER_SLOTS = 30;    
@@ -89,7 +90,6 @@ private:
 
 	bool					GetNextArticle(FileInfo* &pFileInfo, ArticleInfo* &pArticleInfo);
 	void					StartArticleDownload(FileInfo* pFileInfo, ArticleInfo* pArticleInfo, NNTPConnection* pConnection);
-	void					BuildArticleFilename(ArticleDownloader* pArticleDownloader, FileInfo* pFileInfo, ArticleInfo* pArticleInfo);
 	bool					IsDupe(FileInfo* pFileInfo);
 	void					ArticleCompleted(ArticleDownloader* pArticleDownloader);
 	void					DeleteFileInfo(FileInfo* pFileInfo, bool bCompleted);
@@ -121,6 +121,7 @@ public:
 	bool					SetQueueEntryNZBCategory(NZBInfo* pNZBInfo, const char* szCategory);
 	bool					SetQueueEntryNZBName(NZBInfo* pNZBInfo, const char* szName);
 	bool					MergeQueueEntries(NZBInfo* pDestNZBInfo, NZBInfo* pSrcNZBInfo);
+	bool					SplitQueueEntries(FileQueue* pFileList, const char* szName, NZBInfo** pNewNZBInfo);
 	void					DiscardDiskFile(FileInfo* pFileInfo);
 	QueueEditor*			GetQueueEditor() { return &m_QueueEditor; }
 
