@@ -1,7 +1,7 @@
 /*
  *  This file if part of nzbget
  *
- *  Copyright (C) 2007-2011 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 487 $
- * $Date: 2012-10-20 13:06:45 +0200 (Sat, 20 Oct 2012) $
+ * $Revision: 617 $
+ * $Date: 2013-04-06 22:54:00 +0200 (Sat, 06 Apr 2013) $
  *
  */
 
@@ -45,6 +45,7 @@ public:
 		eaFilePauseExtraPars,
 		eaFileSetPriority,
 		eaFileReorder,
+		eaFileSplit,
 		eaGroupMoveOffset,				// move to m_iOffset relative to the current position in queue
 		eaGroupMoveTop,
 		eaGroupMoveBottom,
@@ -96,7 +97,8 @@ private:
 	void					SetNZBCategory(NZBInfo* pNZBInfo, const char* szCategory);
 	void					SetNZBName(NZBInfo* pNZBInfo, const char* szName);
 	bool					CanCleanupDisk(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo);
-	void					MergeGroups(DownloadQueue* pDownloadQueue, ItemList* pItemList);
+	bool					MergeGroups(DownloadQueue* pDownloadQueue, ItemList* pItemList);
+	bool					SplitGroup(DownloadQueue* pDownloadQueue, ItemList* pItemList, const char* szName);
 	void					ReorderFiles(DownloadQueue* pDownloadQueue, ItemList* pItemList);
 	void					SetNZBParameter(NZBInfo* pNZBInfo, const char* szParamString);
 
