@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 554 $
- * $Date: 2013-01-31 21:49:52 +0100 (Thu, 31 Jan 2013) $
+ * $Revision: 633 $
+ * $Date: 2013-04-17 22:21:46 +0200 (Wed, 17 Apr 2013) $
  *
  */
 
@@ -104,7 +104,7 @@ void RemoteServer::Run()
 				m_bTLS);
 			m_pConnection->SetTimeout(g_pOptions->GetConnectionTimeout());
 			m_pConnection->SetSuppressErrors(false);
-			bBind = m_pConnection->Bind() == 0;
+			bBind = m_pConnection->Bind();
 		}
 
 		// Accept connections and store the new Connection
