@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 626 $
- * $Date: 2013-04-09 21:59:32 +0200 (Tue, 09 Apr 2013) $
+ * $Revision: 702 $
+ * $Date: 2013-06-03 22:56:56 +0200 (Mon, 03 Jun 2013) $
  *
  */
 
@@ -33,8 +33,8 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 #include <ctype.h>
-#include <fstream>
 #ifndef WIN32
 #include <unistd.h>
 #endif
@@ -77,7 +77,7 @@ UnpackController::~UnpackController()
 	m_archiveFiles.Clear();
 }
 
-void UnpackController::StartUnpackJob(PostInfo* pPostInfo)
+void UnpackController::StartJob(PostInfo* pPostInfo)
 {
 	UnpackController* pUnpackController = new UnpackController();
 	pUnpackController->m_pPostInfo = pPostInfo;
@@ -125,11 +125,13 @@ void UnpackController::Run()
 
 	snprintf(m_szInfoNameUp, 1024, "Unpack for %s", m_szName); // first letter in upper case
 	m_szInfoNameUp[1024-1] = '\0';
-	
+
+	CheckStateFiles();
+
 #ifndef DISABLE_PARCHECK
-	if (bUnpack && HasBrokenFiles() && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && HasParFiles())
+	if (bUnpack && m_bHasBrokenFiles && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && m_bHasParFiles)
 	{
-		info("%s has broken files", m_szName);
+		PrintMessage(Message::mkInfo, "%s has broken files", m_szName);
 		RequestParCheck(false);
 		m_pPostInfo->SetWorking(false);
 		return;
@@ -138,13 +140,15 @@ void UnpackController::Run()
 
 	if (bUnpack)
 	{
-		CheckArchiveFiles();
+		bool bScanNonStdFiles = m_pPostInfo->GetNZBInfo()->GetRenameStatus() > NZBInfo::rsSkipped ||
+			m_pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSuccess ||
+			!m_bHasParFiles;
+		CheckArchiveFiles(bScanNonStdFiles);
 	}
 
-	if (bUnpack && (m_bHasRarFiles || m_bHasSevenZipFiles || m_bHasSevenZipMultiFiles))
+	if (bUnpack && (m_bHasRarFiles || m_bHasNonStdRarFiles || m_bHasSevenZipFiles || m_bHasSevenZipMultiFiles))
 	{
 		SetInfoName(m_szInfoName);
-		SetDefaultLogKind(g_pOptions->GetProcessLogKind());
 		SetWorkingDir(m_szDestDir);
 
 		PrintMessage(Message::mkInfo, "Unpacking %s", m_szName);
@@ -154,7 +158,7 @@ void UnpackController::Run()
 		m_bUnpackOK = true;
 		m_bUnpackStartError = false;
 
-		if (m_bHasRarFiles)
+		if (m_bHasRarFiles || m_bHasNonStdRarFiles)
 		{
 			ExecuteUnrar();
 		}
@@ -176,14 +180,13 @@ void UnpackController::Run()
 		PrintMessage(Message::mkInfo, (bUnpack ? "Nothing to unpack for %s" : "Unpack for %s skipped"), m_szName);
 
 #ifndef DISABLE_PARCHECK
-		if (bUnpack && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && HasParFiles())
+		if (bUnpack && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && m_bHasParFiles)
 		{
 			RequestParCheck(m_pPostInfo->GetNZBInfo()->GetRenameStatus() <= NZBInfo::rsSkipped);
 		}
 		else
 #endif
 		{
-			m_pPostInfo->SetUnpackStatus(PostInfo::usSkipped);
 			m_pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usSkipped);
 			m_pPostInfo->SetStage(PostInfo::ptQueued);
 		}
@@ -209,19 +212,20 @@ void UnpackController::ExecuteUnrar()
 		szArgs[3] = szPasswordParam;
 	}
 	szArgs[4] = "-o+";
-	szArgs[5] = "*.rar";
+	szArgs[5] = m_bHasNonStdRarFiles ? "*.*" : "*.rar";
 	szArgs[6] = m_szUnpackDir;
 	szArgs[7] = NULL;
 	SetArgs(szArgs, false);
 
 	SetScript(g_pOptions->GetUnrarCmd());
-	SetDefaultKindPrefix("Unrar: ");
+	SetLogPrefix("Unrar");
 
 	m_bAllOKMessageReceived = false;
 	m_eUnpacker = upUnrar;
 	
 	SetProgressLabel("");
 	int iExitCode = Execute();
+	SetLogPrefix(NULL);
 	SetProgressLabel("");
 
 	m_bUnpackOK = iExitCode == 0 && m_bAllOKMessageReceived && !GetTerminated();
@@ -262,14 +266,15 @@ void UnpackController::ExecuteSevenZip(bool bMultiVolumes)
 	SetArgs(szArgs, false);
 
 	SetScript(g_pOptions->GetSevenZipCmd());
-	SetDefaultKindPrefix("7-Zip: ");
 
 	m_bAllOKMessageReceived = false;
 	m_eUnpacker = upSevenZip;
 
 	PrintMessage(Message::mkInfo, "Executing 7-Zip");
+	SetLogPrefix("7-Zip");
 	SetProgressLabel("");
 	int iExitCode = Execute();
+	SetLogPrefix(NULL);
 	SetProgressLabel("");
 
 	m_bUnpackOK = iExitCode == 0 && m_bAllOKMessageReceived && !GetTerminated();
@@ -288,7 +293,6 @@ void UnpackController::Completed()
 	if (m_bUnpackOK && bCleanupSuccess)
 	{
 		PrintMessage(Message::mkInfo, "%s %s", m_szInfoNameUp, "successful");
-		m_pPostInfo->SetUnpackStatus(PostInfo::usSuccess);
 		m_pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usSuccess);
 		m_pPostInfo->GetNZBInfo()->SetUnpackCleanedUpDisk(m_bCleanedUpDisk);
 		m_pPostInfo->SetStage(PostInfo::ptQueued);
@@ -296,7 +300,7 @@ void UnpackController::Completed()
 	else
 	{
 #ifndef DISABLE_PARCHECK
-		if (!m_bUnpackOK && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && !m_bUnpackStartError && !GetTerminated() && HasParFiles())
+		if (!m_bUnpackOK && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && !m_bUnpackStartError && !GetTerminated() && m_bHasParFiles)
 		{
 			RequestParCheck(false);
 		}
@@ -304,7 +308,6 @@ void UnpackController::Completed()
 #endif
 		{
 			PrintMessage(Message::mkError, "%s failed", m_szInfoNameUp);
-			m_pPostInfo->SetUnpackStatus(PostInfo::usFailure);
 			m_pPostInfo->GetNZBInfo()->SetUnpackStatus(NZBInfo::usFailure);
 			m_pPostInfo->SetStage(PostInfo::ptQueued);
 		}
@@ -321,33 +324,30 @@ void UnpackController::RequestParCheck(bool bRename)
 	}
 	else
 	{
-		m_pPostInfo->SetRequestParCheck(PostInfo::rpAll);
+		m_pPostInfo->SetRequestParCheck(true);
 	}
 	m_pPostInfo->SetStage(PostInfo::ptFinished);
 }
 #endif
 
-bool UnpackController::HasParFiles()
-{
-	return ParCoordinator::FindMainPars(m_szDestDir, NULL);
-}
-
-bool UnpackController::HasBrokenFiles()
+void UnpackController::CheckStateFiles()
 {
 	char szBrokenLog[1024];
 	snprintf(szBrokenLog, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, "_brokenlog.txt");
 	szBrokenLog[1024-1] = '\0';
-	return Util::FileExists(szBrokenLog);
+	m_bHasBrokenFiles = Util::FileExists(szBrokenLog);
+
+	m_bHasParFiles = ParCoordinator::FindMainPars(m_szDestDir, NULL);
 }
 
 void UnpackController::CreateUnpackDir()
 {
-	if (strlen(g_pOptions->GetInterDir()) > 0 &&
-		!strncmp(m_szDestDir, g_pOptions->GetInterDir(), strlen(g_pOptions->GetInterDir())))
+	m_bInterDir = strlen(g_pOptions->GetInterDir()) > 0 &&
+		!strncmp(m_szDestDir, g_pOptions->GetInterDir(), strlen(g_pOptions->GetInterDir()));
+	if (m_bInterDir)
 	{
 		m_pPostInfo->GetNZBInfo()->BuildFinalDirName(m_szFinalDir, 1024);
 		m_szFinalDir[1024-1] = '\0';
-		Util::ForceDirectories(m_szFinalDir);
 		snprintf(m_szUnpackDir, 1024, "%s%c%s", m_szFinalDir, PATH_SEPARATOR, "_unpack");
 	}
 	else
@@ -355,17 +355,24 @@ void UnpackController::CreateUnpackDir()
 		snprintf(m_szUnpackDir, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, "_unpack");
 	}
 	m_szUnpackDir[1024-1] = '\0';
-	Util::ForceDirectories(m_szUnpackDir);
+
+	char szErrBuf[1024];
+	if (!Util::ForceDirectories(m_szUnpackDir, szErrBuf, sizeof(szErrBuf)))
+	{
+		error("Could not create directory %s: %s", m_szUnpackDir, szErrBuf);
+	}
 }
 
 
-void UnpackController::CheckArchiveFiles()
+void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 {
 	m_bHasRarFiles = false;
+	m_bHasNonStdRarFiles = false;
 	m_bHasSevenZipFiles = false;
 	m_bHasSevenZipMultiFiles = false;
 
 	RegEx regExRar(".*\\.rar$");
+	RegEx regExRarMultiSeq(".*\\.(r|s)[0-9][0-9]$");
 	RegEx regExSevenZip(".*\\.7z$");
 	RegEx regExSevenZipMulti(".*\\.7z\\.[0-9]*$");
 
@@ -382,14 +389,32 @@ void UnpackController::CheckArchiveFiles()
 			{
 				m_bHasRarFiles = true;
 			}
-			if (regExSevenZip.Match(filename))
+			else if (regExSevenZip.Match(filename))
 			{
 				m_bHasSevenZipFiles = true;
 			}
-			if (regExSevenZipMulti.Match(filename))
+			else if (regExSevenZipMulti.Match(filename))
 			{
 				m_bHasSevenZipMultiFiles = true;
 			}
+			else if (bScanNonStdFiles && !m_bHasNonStdRarFiles && !regExRarMultiSeq.Match(filename))
+			{
+				// Check if file has RAR signature
+				char rarSignature[] = {0x52, 0x61, 0x72, 0x21, 0x1A, 0x07, 0x00};
+				char fileSignature[7];
+
+				FILE* infile;
+				infile = fopen(szFullFilename, "rb");
+				if (infile)
+				{
+					int cnt = (int)fread(fileSignature, 1, sizeof(fileSignature), infile);
+					fclose(infile);
+					if (cnt == sizeof(fileSignature) && !strcmp(rarSignature, fileSignature))
+					{
+						m_bHasNonStdRarFiles = true;
+					}
+				}
+			}
 		}
 	}
 }
@@ -451,7 +476,7 @@ bool UnpackController::Cleanup()
 		{
 			char* szFilename = *it;
 
-			if (!extractedFiles.Exists(szFilename))
+			if (m_bInterDir || !extractedFiles.Exists(szFilename))
 			{
 				char szFullFilename[1024];
 				snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, szFilename);
@@ -480,7 +505,7 @@ bool UnpackController::Cleanup()
 			szFullFilename[1024-1] = '\0';
 
 			if (strcmp(filename, ".") && strcmp(filename, "..") && !Util::DirectoryExists(szFullFilename) 
-				&& regExSevenZip.Match(filename) && !extractedFiles.Exists(filename))
+				&& regExSevenZip.Match(filename) && (m_bInterDir || !extractedFiles.Exists(filename)))
 			{
 				PrintMessage(Message::mkInfo, "Deleting file %s", filename);
 
@@ -562,7 +587,7 @@ bool UnpackController::ReadLine(char* szBuf, int iBufSize, FILE* pStream)
 	return i > 0;
 }
 
-void UnpackController::AddMessage(Message::EKind eKind, bool bDefaultKind, const char* szText)
+void UnpackController::AddMessage(Message::EKind eKind, const char* szText)
 {
 	char szMsgText[1024];
 	strncpy(szMsgText, szText, 1024);
@@ -570,31 +595,39 @@ void UnpackController::AddMessage(Message::EKind eKind, bool bDefaultKind, const
 	
 	// Modify unrar messages for better readability:
 	// remove the destination path part from message "Extracting file.xxx"
-	if (m_eUnpacker == upUnrar && !strncmp(szText, "Extracting  ", 12) &&
-		!strncmp(szText + 12, m_szUnpackDir, strlen(m_szUnpackDir)))
+	if (m_eUnpacker == upUnrar && !strncmp(szText, "Unrar: Extracting  ", 19) &&
+		!strncmp(szText + 19, m_szUnpackDir, strlen(m_szUnpackDir)))
 	{
-		snprintf(szMsgText, 1024, "Extracting %s", szText + 12 + strlen(m_szUnpackDir) + 1);
+		snprintf(szMsgText, 1024, "Unrar: Extracting %s", szText + 19 + strlen(m_szUnpackDir) + 1);
 		szMsgText[1024-1] = '\0';
 	}
 
-	ScriptController::AddMessage(eKind, bDefaultKind, szMsgText);
+	ScriptController::AddMessage(eKind, szMsgText);
 	m_pPostInfo->AppendMessage(eKind, szMsgText);
 
-	if (m_eUnpacker == upUnrar && !strncmp(szMsgText, "Extracting ", 11))
+	if (m_eUnpacker == upUnrar && !strncmp(szMsgText, "Unrar: UNRAR ", 6) &&
+		strstr(szMsgText, " Copyright ") && strstr(szMsgText, " Alexander Roshal"))
 	{
-		SetProgressLabel(szMsgText);
+		// reset start time for a case if user uses unpack-script to do some things
+		// (like sending Wake-On-Lan message) before executing unrar
+		m_pPostInfo->SetStageTime(time(NULL));
 	}
 
-	if (m_eUnpacker == upUnrar && !strncmp(szText, "Extracting from ", 16))
+	if (m_eUnpacker == upUnrar && !strncmp(szMsgText, "Unrar: Extracting ", 18))
 	{
-		const char *szFilename = szText + 16;
+		SetProgressLabel(szMsgText + 7);
+	}
+
+	if (m_eUnpacker == upUnrar && !strncmp(szText, "Unrar: Extracting from ", 23))
+	{
+		const char *szFilename = szText + 23;
 		debug("Filename: %s", szFilename);
 		m_archiveFiles.push_back(strdup(szFilename));
-		SetProgressLabel(szText);
+		SetProgressLabel(szText + 7);
 	}
 
-	if ((m_eUnpacker == upUnrar && !strncmp(szText, "All OK", 6)) ||
-		(m_eUnpacker == upSevenZip && !strncmp(szText, "Everything is Ok", 16)))
+	if ((m_eUnpacker == upUnrar && !strncmp(szText, "Unrar: All OK", 13)) ||
+		(m_eUnpacker == upSevenZip && !strncmp(szText, "7-Zip: Everything is Ok", 23)))
 	{
 		m_bAllOKMessageReceived = true;
 	}
@@ -615,7 +648,7 @@ void UnpackController::SetProgressLabel(const char* szProgressLabel)
 }
 
 
-void MoveController::StartMoveJob(PostInfo* pPostInfo)
+void MoveController::StartJob(PostInfo* pPostInfo)
 {
 	MoveController* pMoveController = new MoveController();
 	pMoveController->m_pPostInfo = pPostInfo;
@@ -640,9 +673,6 @@ void MoveController::Run()
 	szInfoName[1024-1] = '\0';
 	SetInfoName(szInfoName);
 
-	SetDefaultKindPrefix("Move: ");
-	SetDefaultLogKind(g_pOptions->GetProcessLogKind());
-
 	strncpy(m_szInterDir, m_pPostInfo->GetNZBInfo()->GetDestDir(), 1024);
 	m_szInterDir[1024-1] = '\0';
 
@@ -678,10 +708,14 @@ void MoveController::Run()
 
 bool MoveController::MoveFiles()
 {
+	char szErrBuf[1024];
+	if (!Util::ForceDirectories(m_szDestDir, szErrBuf, sizeof(szErrBuf)))
+	{
+		error("Could not create directory %s: %s", m_szDestDir, szErrBuf);
+		return false;
+	}
+
 	bool bOK = true;
-
-	bOK = Util::ForceDirectories(m_szDestDir);
-
 	DirBrowser dir(m_szInterDir);
 	while (const char* filename = dir.Next())
 	{
@@ -717,3 +751,138 @@ bool MoveController::MoveFiles()
 
 	return bOK;
 }
+
+
+void CleanupController::StartJob(PostInfo* pPostInfo)
+{
+	CleanupController* pCleanupController = new CleanupController();
+	pCleanupController->m_pPostInfo = pPostInfo;
+	pCleanupController->SetAutoDestroy(false);
+
+	pPostInfo->SetPostThread(pCleanupController);
+
+	pCleanupController->Start();
+}
+
+void CleanupController::Run()
+{
+	// the locking is needed for accessing the members of NZBInfo
+	g_pDownloadQueueHolder->LockQueue();
+
+	char szNZBName[1024];
+	strncpy(szNZBName, m_pPostInfo->GetNZBInfo()->GetName(), 1024);
+	szNZBName[1024-1] = '\0';
+
+	char szInfoName[1024];
+	snprintf(szInfoName, 1024, "cleanup for %s", m_pPostInfo->GetNZBInfo()->GetName());
+	szInfoName[1024-1] = '\0';
+	SetInfoName(szInfoName);
+
+	strncpy(m_szDestDir, m_pPostInfo->GetNZBInfo()->GetDestDir(), 1024);
+	m_szDestDir[1024-1] = '\0';
+
+	bool bInterDir = strlen(g_pOptions->GetInterDir()) > 0 &&
+		!strncmp(m_szDestDir, g_pOptions->GetInterDir(), strlen(g_pOptions->GetInterDir()));
+	if (bInterDir)
+	{
+		m_pPostInfo->GetNZBInfo()->BuildFinalDirName(m_szFinalDir, 1024);
+		m_szFinalDir[1024-1] = '\0';
+	}
+	else
+	{
+		m_szFinalDir[0] = '\0';
+	}
+
+	g_pDownloadQueueHolder->UnlockQueue();
+
+	info("Cleaning up %s", szNZBName);
+
+	bool bDeleted = false;
+	bool bOK = Cleanup(m_szDestDir, &bDeleted);
+
+	if (bOK && m_szFinalDir[0] != '\0')
+	{
+		bool bDeleted2 = false;
+		bOK = Cleanup(m_szFinalDir, &bDeleted2);
+		bDeleted = bDeleted || bDeleted2;
+	}
+
+	szInfoName[0] = 'C'; // uppercase
+
+	if (bOK && bDeleted)
+	{
+		info("%s successful", szInfoName);
+		m_pPostInfo->GetNZBInfo()->SetCleanupStatus(NZBInfo::csSuccess);
+	}
+	else if (bOK)
+	{
+		info("Nothing to cleanup for %s", szNZBName);
+		m_pPostInfo->GetNZBInfo()->SetCleanupStatus(NZBInfo::csSuccess);
+	}
+	else
+	{
+		error("%s failed", szInfoName);
+		m_pPostInfo->GetNZBInfo()->SetCleanupStatus(NZBInfo::csFailure);
+	}
+
+	m_pPostInfo->SetStage(PostInfo::ptQueued);
+	m_pPostInfo->SetWorking(false);
+}
+
+bool CleanupController::Cleanup(const char* szDestDir, bool *bDeleted)
+{
+	*bDeleted = false;
+	bool bOK = true;
+
+	ExtList extList;
+	
+	// split ExtCleanupDisk into tokens and create a list
+	char* szExtCleanupDisk = strdup(g_pOptions->GetExtCleanupDisk());
+	
+	char* saveptr;
+	char* szExt = strtok_r(szExtCleanupDisk, ",; ", &saveptr);
+	while (szExt)
+	{
+		extList.push_back(szExt);
+		szExt = strtok_r(NULL, ",; ", &saveptr);
+	}
+	
+	DirBrowser dir(szDestDir);
+	while (const char* filename = dir.Next())
+	{
+		// check file extension
+		
+		int iFilenameLen = strlen(filename);
+		bool bDeleteIt = false;
+		for (ExtList::iterator it = extList.begin(); it != extList.end(); it++)
+		{
+			const char* szExt = *it;
+			int iExtLen = strlen(szExt);
+			if (iFilenameLen >= iExtLen && !strcasecmp(szExt, filename + iFilenameLen - iExtLen))
+			{
+				bDeleteIt = true;
+				break;
+			}
+		}
+
+		if (bDeleteIt)
+		{
+			char szFullFilename[1024];
+			snprintf(szFullFilename, 1024, "%s%c%s", szDestDir, PATH_SEPARATOR, filename);
+			szFullFilename[1024-1] = '\0';
+
+			PrintMessage(Message::mkInfo, "Deleting file %s", filename);
+			if (remove(szFullFilename) != 0)
+			{
+				PrintMessage(Message::mkError, "Could not delete file %s! Errcode: %i", szFullFilename, errno);
+				bOK = false;
+			}
+
+			*bDeleted = true;
+		}
+	}
+
+	free(szExtCleanupDisk);
+
+	return bOK;
+}
