@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 548 $
- * $Date: 2013-01-21 22:19:04 +0100 (Mon, 21 Jan 2013) $
+ * $Revision: 629 $
+ * $Date: 2013-04-15 22:06:05 +0200 (Mon, 15 Apr 2013) $
  *
  */
 
@@ -27,19 +27,7 @@
 #define XMLRPC_H
 
 #include "Connection.h"
-
-class StringBuilder
-{
-private:
-	char*				m_szBuffer;
-	int					m_iBufferSize;
-	int					m_iUsedSize;
-public:
-						StringBuilder();
-						~StringBuilder();
-	void				Append(const char* szStr);
-	const char*			GetBuffer() { return m_szBuffer; }
-};
+#include "Util.h"
 
 class XmlCommand;
 
@@ -282,7 +270,12 @@ class SaveConfigXmlCommand: public XmlCommand
 {
 public:
 	virtual void		Execute();
-	void				Save(const char *szFilename);
+};
+
+class ConfigTemplatesXmlCommand: public XmlCommand
+{
+public:
+	virtual void		Execute();
 };
 
 #endif
